## begin license ##
#
#    Meresco Core is an open-source library containing components to build
#    searchengines, repositories and archives.
#    Copyright (C) 2007-2009 Seek You Too (CQ2) http://www.cq2.nl
#    Copyright (C) 2007-2009 SURF Foundation. http://www.surf.nl
#    Copyright (C) 2007-2009 Stichting Kennisnet Ict op school.
#       http://www.kennisnetictopschool.nl
#    Copyright (C) 2007 SURFnet. http://www.surfnet.nl
#
#    This file is part of Meresco Core.
#
#    Meresco Core is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Meresco Core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Meresco Core; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

from meresco.components import XmlCompose
from xml.sax.saxutils import escape as xmlEscape
from urllib import quote as urlQuote

RSS_TEMPLATE = """<item>
    <pubDate>%(pubDate)s</pubDate>
    <title>%(title)s</title>
    <dc:creator>%(creator)s</dc:creator>
    <description>%(description)s</description>
    <link>%(link)s</link>
    <guid>%(link)s</guid>
</item>"""

class RssItem(XmlCompose):
    def __init__(self, nsMap, pubDate, linkTemplate, **linkFields):
        XmlCompose.__init__(self,
            template="ignored",
            nsMap=nsMap,
            pubDate=pubDate,
            **linkFields)
        self._linkTemplate = linkTemplate
        assertLinkTemplate(linkTemplate, linkFields)

    def createRecord(self, dataDictionary):
        try:
            link = self._linkTemplate % dict(((k, urlQuote(v)) for k,v in dataDictionary.items()))
        except KeyError:
            link = ''
        recordId = dataDictionary.get('uploadId', '')
        record = self.any.createRecord(recordId)
        description = ''
        creator = ''
        if 'Author(s)' in record:
            description = record['Author(s)']
            creator = record['Author(s)']
        if 'Publication date' in record:
            description += ' (' + record['Publication date'] + ')'
        if 'Source' in record:
            description += '<br/>In: ' + record['Source']
        if 'Abstract' in record:
            description += '<br/><br/>' + record['Abstract']
        rssData = {
            'link': xmlEscape(link),
            'pubDate': xmlEscape(dataDictionary.get('pubDate', '')),
            'description': xmlEscape(description),
            'title': xmlEscape(record['Title']),
            'creator': creator
        }
        return str(RSS_TEMPLATE % rssData)

def assertLinkTemplate(linkTemplate, linkFields):
    try:
        linkTemplate % dict(((k,'value') for k in linkFields.keys()))
    except KeyError, e:
        givenArguments = len(linkFields) + len(['self', 'nsMap', 'title', 'description', 'linkTemplate'])
        raise TypeError("__init__() takes at least %s arguments (%s given, missing %s)" % (givenArguments + 1, givenArguments, str(e)))
